"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  RemixFormProvider: () => RemixFormProvider,
  createFormData: () => createFormData,
  getFormDataFromSearchParams: () => getFormDataFromSearchParams,
  getValidatedFormData: () => getValidatedFormData,
  parseFormData: () => parseFormData,
  useRemixForm: () => useRemixForm,
  useRemixFormContext: () => useRemixFormContext,
  validateFormData: () => validateFormData
});
module.exports = __toCommonJS(src_exports);

// src/utilities/index.ts
var tryParseJSON = (jsonString) => {
  try {
    const json = JSON.parse(jsonString);
    return json;
  } catch (e) {
    return jsonString;
  }
};
var generateFormData = (formData, preserveStringified = false) => {
  const outputObject = {};
  for (const [key, value] of formData.entries()) {
    const data = preserveStringified ? value : tryParseJSON(value.toString());
    const keyParts = key.split(".");
    let currentObject = outputObject;
    for (let i = 0; i < keyParts.length - 1; i++) {
      const keyPart = keyParts[i];
      if (!currentObject[keyPart]) {
        currentObject[keyPart] = /^\d+$/.test(keyParts[i + 1]) ? [] : {};
      }
      currentObject = currentObject[keyPart];
    }
    const lastKeyPart = keyParts[keyParts.length - 1];
    const lastKeyPartIsArray = /\[\d*\]$|\[\]$/.test(lastKeyPart);
    if (lastKeyPartIsArray) {
      const key2 = lastKeyPart.replace(/\[\d*\]$|\[\]$/, "");
      if (!currentObject[key2]) {
        currentObject[key2] = [];
      }
      currentObject[key2].push(data);
    }
    if (!lastKeyPartIsArray) {
      if (/^\d+$/.test(lastKeyPart)) {
        currentObject.push(data);
      } else {
        currentObject[lastKeyPart] = data;
      }
    }
  }
  return outputObject;
};
var getFormDataFromSearchParams = (request, preserveStringified = false) => {
  const searchParams = new URL(request.url).searchParams;
  return generateFormData(searchParams, preserveStringified);
};
var isGet = (request) => request.method === "GET" || request.method === "get";
var getValidatedFormData = async (request, resolver) => {
  const data = isGet(request) ? getFormDataFromSearchParams(request) : await parseFormData(request);
  const validatedOutput = await validateFormData(data, resolver);
  return { ...validatedOutput, receivedValues: data };
};
var validateFormData = async (data, resolver) => {
  const dataToValidate = data instanceof FormData ? Object.fromEntries(data) : data;
  const { errors, values } = await resolver(
    dataToValidate,
    {},
    { shouldUseNativeValidation: false, fields: {} }
  );
  if (Object.keys(errors).length > 0) {
    return { errors, data: void 0 };
  }
  return { errors: void 0, data: values };
};
var createFormData = (data) => {
  const formData = new FormData();
  if (!data) {
    return formData;
  }
  Object.entries(data).map(([key, value]) => {
    if (value instanceof FileList) {
      for (let i = 0; i < value.length; i++) {
        formData.append(key, value[i]);
      }
      return;
    }
    if (value instanceof File || value instanceof Blob) {
      formData.append(key, value);
    } else
      formData.append(key, JSON.stringify(value));
  });
  return formData;
};
var parseFormData = async (request, preserveStringified = false) => {
  const formData = await request.formData();
  return generateFormData(formData, preserveStringified);
};
var mergeErrors = (frontendErrors, backendErrors, validKeys = [], depth = 0) => {
  if (!backendErrors) {
    return frontendErrors;
  }
  for (const [key, rightValue] of Object.entries(backendErrors)) {
    if (!validKeys.includes(key.toString()) && validKeys.length && depth === 0) {
      continue;
    }
    if (typeof rightValue === "object" && !Array.isArray(rightValue)) {
      if (!frontendErrors[key]) {
        frontendErrors[key] = {};
      }
      mergeErrors(frontendErrors[key], rightValue, validKeys, depth + 1);
    } else if (rightValue) {
      frontendErrors[key] = rightValue;
    }
  }
  return frontendErrors;
};

// src/hook/index.tsx
var import_react = __toESM(require("react"), 1);
var import_react2 = require("@remix-run/react");
var import_react_hook_form = require("react-hook-form");
var import_react_hook_form2 = require("react-hook-form");
var useRemixForm = ({
  submitHandlers,
  submitConfig,
  submitData,
  fetcher,
  ...formProps
}) => {
  var _a, _b, _c, _d;
  const [isSubmittedSuccessfully, setIsSubmittedSuccessfully] = import_react.default.useState(false);
  const actionSubmit = (0, import_react2.useSubmit)();
  const actionData = (0, import_react2.useActionData)();
  const submit = (_a = fetcher == null ? void 0 : fetcher.submit) != null ? _a : actionSubmit;
  const data = (_b = fetcher == null ? void 0 : fetcher.data) != null ? _b : actionData;
  const methods = (0, import_react_hook_form2.useForm)(formProps);
  const navigation = (0, import_react2.useNavigation)();
  const isSubmittingForm = navigation.state !== "idle" || fetcher && fetcher.state !== "idle";
  const onSubmit = (data2) => {
    setIsSubmittedSuccessfully(true);
    const formData = createFormData({ ...data2, ...submitData });
    submit(formData, {
      method: "post",
      ...submitConfig
    });
  };
  const values = methods.getValues();
  const validKeys = Object.keys(values);
  const onInvalid = () => {
  };
  const formState = methods.formState;
  const {
    dirtyFields,
    isDirty,
    isSubmitSuccessful,
    isSubmitted,
    isSubmitting,
    isValid,
    isValidating,
    touchedFields,
    submitCount,
    errors,
    isLoading,
    disabled
  } = formState;
  const formErrors = mergeErrors(
    errors,
    (data == null ? void 0 : data.errors) ? data.errors : data,
    validKeys
  );
  return {
    ...methods,
    handleSubmit: methods.handleSubmit(
      (_c = submitHandlers == null ? void 0 : submitHandlers.onValid) != null ? _c : onSubmit,
      (_d = submitHandlers == null ? void 0 : submitHandlers.onInvalid) != null ? _d : onInvalid
    ),
    reset: (values2) => {
      setIsSubmittedSuccessfully(false);
      methods.reset(values2);
    },
    register: (name, options) => {
      var _a2, _b2;
      return {
        ...methods.register(name, options),
        ...!(options == null ? void 0 : options.disableProgressiveEnhancement) && {
          defaultValue: (_b2 = (_a2 = data == null ? void 0 : data.defaultValues) == null ? void 0 : _a2[name]) != null ? _b2 : ""
        }
      };
    },
    formState: {
      disabled,
      dirtyFields,
      isDirty,
      isSubmitSuccessful: isSubmittedSuccessfully || isSubmitSuccessful,
      isSubmitted,
      isSubmitting: isSubmittingForm || isSubmitting,
      isValid,
      isValidating,
      touchedFields,
      submitCount,
      isLoading,
      errors: formErrors
    }
  };
};
var RemixFormProvider = ({
  children,
  ...props
}) => {
  return /* @__PURE__ */ import_react.default.createElement(import_react_hook_form2.FormProvider, { ...props }, children);
};
var useRemixFormContext = () => {
  const methods = (0, import_react_hook_form.useFormContext)();
  return {
    ...methods,
    handleSubmit: methods.handleSubmit
  };
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  RemixFormProvider,
  createFormData,
  getFormDataFromSearchParams,
  getValidatedFormData,
  parseFormData,
  useRemixForm,
  useRemixFormContext,
  validateFormData
});
